import os
import random
import json
import xml.etree.ElementTree as ET

# Don't change this line! It prevents the XML parser from
# adding a namespace to the tags in the SVG file. This
# keeps them easy to read.
ET.register_namespace('', 'http://www.w3.org/2000/svg')


class Emoji:

    @staticmethod
    def from_svg(svg_content):
        """Given the contents of an SVG as text,
        parse the SVG and store the tree create a
        dictionary that maps the class names to sets that
        stores the elements of the SVG with that class.

        The SVG has a root <svg> tag, and this root
        will contain one or more <g> tags. Each of these
        <g> tags will have zero or more child elements of
        several different kinds. Each of these child elements
        will have a class attribute that specifies the
        group to which it belongs: face, decoration, mouth, or eyes.
        """
        pass

    def __init__(self, groups):
        self.groups = groups

    def to_svg(self, filename):
        svg = ET.Element('svg', attrib={'id': 'emoji', 'viewBox': '0 0 72 72'})
        for group in self.groups:
            g = ET.Element('g', attrib={'id': group})
            for element in self.groups[group]:
                g.append(element)
            svg.append(g)
        with open(filename, 'w') as f:
            f.write(ET.tostring(svg, encoding='unicode'))
        return ET.tostring(svg, encoding='unicode')

    def __getitem__(self, key):
        """Returns the set of elements in the group with the given key.

        If the key is not in the groups dictionary, this method should
        return an empty set.
        """
        pass

    def __setitem__(self, key, value):
        """Sets the set of elements in the group with the given key.

        This will totally overwrite any existing elements in the group.
        If you want to add elements to a group, then you'll want to use
        the + operator instead.

        It is possible to use a key that is not already in the groups
        dictionary (i.e. not face, decoration, mouth, or eyes).
        """
        pass

    def __add__(self, other):
        """The input to this method can either be another Emoji object.

        or a dictionary that maps group names to sets of elements.

        Independent of whether the input is another Emoji or a dictionary of
        groups, this method should initialize a new Emoji object. This new
        Emoji should have as its groups attribute a dictionary storing
        the unions of the elements in each group. That is,
        include all graphical elements in the groups of both emojis.
        The groups themselves should still be preserved.
        """
        pass

    def __sub__(self, other):
        """The input to this method is a tuple of strings.

        This method returns a new Emoji.
        Each string in the input tuple is the name of a group
        that should be empty in the new Emoji. Any groups not present
        in the input tuple should be copied as-is into the new Emoji.
        """
        pass


class EmojiLibrary:
    def __init__(self, directory_path):
        self.emojis = {}
        self.load_emojis(directory_path)

    def load_emojis(self, directory_path):
        """Given a path to a directory, this method will load all the emojis
        in that directory into the library.

        The directory will contain a file called 'emojis.json' that
        contains a list of dictionaries.
        Each dictionary will have a 'name' key and a 'hex' key.
        The 'name' key will be the name of the emoji, and the 'hex'
        key will be the filename of the SVG file that contains the emoji.
        (You will need to add the .svg extension to the filename)

        This method should modify self.emojis so that the
        dictionary contains a mapping from each emoji name to an Emoji object.
        """
        pass

    def get_emoji(self, name):
        return self.emojis.get(name)

    def get_random_emoji(self):
        return random.choice(list(self.emojis.values()))

    def get_random_subset(self, count):
        return random.sample(
            list(self.emojis.values()),
            min(count, len(self.emojis)))

    def generate_amalgam(self):
        emojis = self.get_random_subset(4)
        eyes = emojis[0]["eyes"]
        face = emojis[1]["face"]
        mouth = emojis[2]["mouth"]
        decoration = emojis[3]["decoration"]

        new_emoji = Emoji({
            "eyes": eyes,
            "face": face,
            "mouth": mouth,
            "decoration": decoration
        })
        return new_emoji
