"""
🚨 🚨 🚨 🚨 🚨 🚨 🚨 🚨 🚨 🚨 🚨 🚨
DON'T TOUCH ANYTHING IN THIS FILE!
🚨 🚨 🚨 🚨 🚨 🚨 🚨 🚨 🚨 🚨 🚨 🚨
"""


from http.server import HTTPServer, SimpleHTTPRequestHandler
import os
from datetime import datetime
from urllib.parse import quote


class SVGPreviewDirectoryHandler(SimpleHTTPRequestHandler):
    def list_directory(self, path):
        try:
            list = os.listdir(path)
        except OSError:
            self.send_error(404, "No permission to list directory")
            return None

        r = []
        displaypath = self.path
        r.append('<!DOCTYPE html>')
        r.append('<html>\n<head>')
        r.append('<meta charset="utf-8">')
        r.append(f'<title>Directory listing for {displaypath}</title>')
        r.append('<style>')
        r.append('''
            body {
                font-family: system-ui, -apple-system, sans-serif;
                max-width: 1200px;
                margin: 20px auto;
                padding: 0 20px;
            }
            table { width: 100%; border-collapse: collapse; }
            th, td { text-align: left; padding: 8px;
                     border-bottom: 1px solid #ddd; }
            th { background-color: #f5f5f5; position: sticky; top: 0; }
            a { color: #2563eb; text-decoration: none; }
            a:hover { text-decoration: underline; }
            .size { text-align: right; }
            .date { color: #666; }
            .preview-cell {
                max-width: 200px;
                overflow: hidden;
            }
            .preview-cell svg {
                max-width: 150px;
                max-height: 100px;
                display: block;
            }
            .parent-link {
                display: inline-block;
                margin-bottom: 20px;
                padding: 8px 16px;
                background: #f5f5f5;
                border-radius: 4px;
            }
            .parent-link::before {
                content: "↑ ";
            }
        ''')
        r.append('</style>')
        r.append('</head>')
        r.append('<body>')

        # Add parent directory link at the top if not in root
        if displaypath != '/':
            r.append('<a href=".." class="parent-link">Parent Directory</a>')

        r.append(f'<h1>Directory listing for {displaypath}</h1>')
        r.append('<table>')
        r.append(
            '<tr><th>Name</th><th>Size</th><th>Last Modified</th>'
            '<th>Preview</th></tr>'
        )

        if displaypath != '/':
            r.append(
                '<tr><td><a href="..">..</a></td><td>'
                '</td><td></td><td></td></tr>'
            )

        for name in sorted(list):
            fullname = os.path.join(path, name)
            displayname = linkname = name

            # Get file info
            stat_info = os.stat(fullname)
            size = stat_info.st_size
            last_modified = datetime.fromtimestamp(stat_info.st_mtime)

            if os.path.isdir(fullname):
                displayname = name + "/"
                linkname = name + "/"
                size_str = "-"
                preview_cell = ""
            else:
                size_str = self._format_size(size)
                preview_cell = self._generate_preview_cell(fullname, size)

            r.append('<tr>')
            r.append(f'<td><a href="{quote(linkname)}">{displayname}</a></td>')
            r.append(f'<td class="size">{size_str}</td>')
            r.append(
                '<td class="date">'
                f'{last_modified.strftime("%Y-%m-%d %H:%M")}</td>')
            r.append(f'<td class="preview-cell">{preview_cell}</td>')
            r.append('</tr>')

        r.append('</table>')
        r.append('</body>\n</html>\n')

        encoded = '\n'.join(r).encode('utf-8')
        self.send_response(200)
        self.send_header("Content-type", "text/html; charset=utf-8")
        self.send_header("Content-Length", str(len(encoded)))
        self.end_headers()
        self.wfile.write(encoded)
        return None

    def _format_size(self, size):
        """Format file size in human readable format."""
        for unit in ['B', 'KB', 'MB', 'GB']:
            if size < 1024:
                return f"{size:.1f} {unit}"
            size /= 1024
        return f"{size:.1f} TB"

    def _generate_preview_cell(self, fullname, size):
        """Generate preview cell content for SVG files only."""
        if not fullname.lower().endswith('.svg'):
            return ""

        if size > 5_000_000:  # Skip files larger than 5MB
            return "<span>File too large for preview</span>"

        try:
            with open(fullname, 'r', encoding='utf-8') as f:
                svg_content = f.read()
            return svg_content
        except (IOError, UnicodeDecodeError):
            return "<span>Preview unavailable</span>"

        return ""


if __name__ == '__main__':
    server_address = ('', 8000)
    httpd = HTTPServer(server_address, SVGPreviewDirectoryHandler)
    print("\n"*80)
    print('Server running on port 8000...')
    print('🚨 🚨 Just leave me running! 🚨 🚨')
    print('🔍 🔎 To view your SVGs, click the SVG Viewer button above.🔍 🔎')
    httpd.serve_forever()
